"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const MongoDB = require("mongodb");
class Collection {
    constructor(collection, modelConstructor) {
        this.collection = collection;
        this.modelConstructor = modelConstructor;
    }
    create() {
        let model = new this.modelConstructor(this);
        model._collection = this;
        return model;
    }
    async insert(model) {
        return await this.collection.insertOne(model.toServer());
    }
    async update(id, model, overwrite) {
        if (typeof id === 'string') {
            id = new MongoDB.ObjectId(id);
        }
        return await this.collection.updateOne({ _id: id }, {
            $set: overwrite ?
                model.toServer() :
                model.diff()
        });
    }
    async findOne(query) {
        let result = await this.collection.findOne(query);
        if (result) {
            let model = this.create();
            model.fromServer(result);
            return model;
        }
    }
    async get(id) {
        if (typeof id === 'string') {
            id = new MongoDB.ObjectId(id);
        }
        let result = await this.collection.findOne({
            _id: id
        }, {});
        if (result) {
            let model = this.create();
            model.fromServer(result);
            return model;
        }
    }
    async list(params) {
        params = params || {};
        var find = params.find || {};
        var offset = params.offset;
        var limit = params.limit;
        var sort = params.sort;
        let query = this.collection.find(find);
        let count = await query.count();
        if (limit !== 0) {
            query = query.skip(offset || 0)
                .limit(limit);
        }
        if (sort) {
            query = query.sort(sort);
        }
        let array = await query.toArray();
        let data = array.map((item) => {
            let model = this.create();
            model.fromServer(item);
            return model;
        });
        return {
            count: count,
            results: data
        };
    }
    delete(id) {
        if (typeof id === 'string') {
            id = new MongoDB.ObjectId(id);
        }
        return this.collection.deleteOne({
            _id: id
        });
    }
}
exports.default = Collection;
//# sourceMappingURL=Collection.js.map