"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const MongoDB = require("mongodb");
const Decorators_1 = require("./Decorators");
class Model {
    constructor(collection) {
        Object.defineProperties(this, propertyDefinitions);
        this._collection = collection;
    }
    reset() {
        let configs = this.getConfigs();
        Object.keys(configs).forEach(key => {
            this[key] = this._baseData[key];
        });
    }
    update() {
        let configs = this.getConfigs();
        if (!this._baseData) {
            this._baseData = {};
        }
        Object.keys(configs).forEach(key => {
            this._baseData[key] = this[key];
        });
    }
    diff() {
        let keys = this.getKeys();
        let diff = {};
        let baseData = this._baseData;
        let differentKeys = keys.forEach(key => {
            if (this[key] !== baseData[key]) {
                diff[key] = this[key];
            }
        });
        return diff;
    }
    getKeys() {
        if (this._modelDefinition) {
            return this._modelDefinition.getKeys();
        }
        else {
            return [];
        }
    }
    getConfig(key) {
        if (this._modelDefinition) {
            return this._modelDefinition.getConfig(key);
        }
    }
    getConfigs() {
        if (this._modelDefinition) {
            return this._modelDefinition.getConfigs();
        }
        else {
            return {};
        }
    }
    validate() {
        let output = [];
        let configs = this.getConfigs();
        Object.keys(configs).forEach(key => {
            let config = configs[key];
            if (config && config.required) {
                if (!this[key]) {
                    output.push(key);
                }
            }
        });
        return output;
    }
    build(collectionFactory, references) {
        let referenceHash = {};
        switch (typeof references) {
            case 'string':
                referenceHash[references] = true;
                break;
            case 'object':
                if (references instanceof Array) {
                    references.forEach(reference => {
                        referenceHash[reference] = true;
                    });
                }
                break;
        }
        let configs = this.getConfigs();
        Object.keys(configs).forEach(key => {
            let config = configs[key];
            if (config.reference && referenceHash[key]) {
                if (config.reference.collection && config.reference.id) {
                    let collection = collectionFactory.getCollection(config.reference.collection);
                    if (collection) {
                    }
                }
            }
        });
    }
    fromClient(data) {
        data = data || {};
        this._baseData = {};
        let configs = this.getConfigs();
        Object.keys(configs).forEach(key => {
            let config = configs[key];
            if ((typeof config.default !== 'undefined') && (typeof data[key] === 'undefined')) {
                let value = config.default;
                this[key] = value;
                this._baseData[key] = value;
            }
            else {
                let value = config.fromClient ? config.fromClient(data[key]) : data[key];
                this[key] = value;
                this._baseData[key] = value;
            }
        });
    }
    toClient(hide, references) {
        let output = {};
        let configs = this.getConfigs();
        let useReferences;
        let referenceHash = {};
        switch (typeof references) {
            case 'boolean':
                useReferences = references;
                break;
            case 'string':
                useReferences = true;
                referenceHash[references] = true;
                break;
            case 'object':
                if (references instanceof Array) {
                    useReferences = true;
                    references.forEach(reference => {
                        referenceHash[reference] = true;
                    });
                }
                break;
        }
        Object.keys(configs).forEach(key => {
            let config = configs[key];
            if (!hide || !config.hide) {
                if (config.reference) {
                }
                output[key] = config.toClient ? config.toClient(this[key]) : this[key];
            }
        });
        return output;
    }
    fromServer(data) {
        data = data || {};
        this._baseData = {};
        let configs = this.getConfigs();
        Object.keys(configs).forEach(key => {
            let config = configs[key];
            if ((typeof config.default !== 'undefined') && (typeof data[key] === 'undefined')) {
                let value = config.default;
                this[key] = value;
                this._baseData[key] = value;
            }
            else {
                let value = config.fromServer ? config.fromServer(data[key]) : data[key];
                this[key] = value;
                this._baseData[key] = value;
            }
        });
    }
    toServer(hide, references) {
        let output = {};
        let configs = this.getConfigs();
        let useReferences;
        let referenceHash = {};
        switch (typeof references) {
            case 'boolean':
                useReferences = references;
                break;
            case 'string':
                useReferences = true;
                referenceHash[references] = true;
                break;
            case 'object':
                if (references instanceof Array) {
                    useReferences = true;
                    references.forEach(reference => {
                        referenceHash[reference] = true;
                    });
                }
                break;
        }
        Object.keys(configs).forEach(key => {
            let config = configs[key];
            if (!hide || !config.hide) {
                if (config.reference) {
                }
                output[key] = config.toServer ? config.toServer(this[key]) : this[key];
            }
        });
        return output;
    }
    toJSON() {
        return this.toClient(true);
    }
    async save(overwrite) {
        if (this._collection) {
            this.beforeSave();
            if (this._id) {
                this.beforeUpdate(overwrite);
                let result = await this._collection.update(this._id, this, overwrite);
                this.update();
                return result;
            }
            else {
                this.beforeInsert();
                let result = await this._collection.insert(this);
                this._id = result.insertedId;
                this.update();
                return result;
            }
        }
    }
    async delete() {
        if (this._collection) {
            if (this._id) {
                this.beforeDelete();
                await this._collection.delete(this._id);
                this._id = undefined;
            }
        }
    }
    beforeSave() {
    }
    beforeInsert() {
    }
    beforeUpdate(overwrite) {
    }
    beforeDelete() {
    }
}
__decorate([
    Decorators_1.prop({
        fromClient: (value) => {
            if (typeof value === 'string') {
                return new MongoDB.ObjectId(value);
            }
            else {
                return value;
            }
        },
        toClient: (value) => {
            if (value && value.toHexString) {
                return value.toHexString();
            }
            else {
                return value;
            }
        }
    }),
    __metadata("design:type", MongoDB.ObjectId)
], Model.prototype, "_id", void 0);
exports.default = Model;
let propertyDefinitions = {
    _baseData: {
        enumerable: false,
        writable: true,
        configurable: true
    },
    _collection: {
        enumerable: false,
        writable: true,
        configurable: true
    }
};
//# sourceMappingURL=Model.js.map