import { IClientData } from '../interfaces/IData';
import Model from './Model';
export interface IPropertyConfig<T, U, V, W extends keyof T, X extends keyof U, Y extends keyof V> {
    key?: W;
    clientKey?: X;
    serverKey?: Y;
    type?: any;
    required?: boolean;
    default?: T[W];
    hide?: boolean;
    reference?: {
        collection: string;
        id: string;
    };
    fromClient?: (value: U[X]) => T[W];
    toClient?: (value: T[W]) => U[X];
    fromServer?: (value: V[Y]) => T[W];
    toServer?: (value: T[W]) => V[Y];
    validation?: (value: T[W]) => boolean;
    minLength?: number;
    maxLength?: number;
    trim?: boolean;
    minimum?: number;
    maximum?: number;
}
export interface IModel {
    _modelDefinition: ModelDefinition;
}
export default class ModelDefinition {
    parent: ModelDefinition;
    propertyConfigs: {
        [index: string]: IPropertyConfig<any, any, any, any, any, any>;
    };
    constructor(parent?: ModelDefinition);
    getKeys(): any;
    getConfigs(): {
        [index: string]: IPropertyConfig<any, any, any, any, any, any>;
    };
    getConfig<T extends IModel>(key: keyof T): IPropertyConfig<any, any, any, any, any, any>;
    addConfig<T extends IModel>(key: keyof T, config: IPropertyConfig<any, any, any, any, any, any>): void;
    static getModelDefinition<T extends IClientData = any>(target: Model<T>): ModelDefinition;
}
